package autosfx;

import javafx.scene.canvas.Canvas;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.paint.Color;

public class AutoCanvas extends Canvas {
    private final int w;//800
    private final int h;//800
    private final int liczbaPol;//100
    private final int szer;//szerokosc komorki 8
    private final int len;
    private final Ksztalt ksztalt;
    private final Color kolor;
    private final int[][] autos;

    public AutoCanvas(int autoNumber, int[] kombStart, Ksztalt ksztalt, Color kolor, int w, int h, int liczbaPol, int szer) {
        this.ksztalt = ksztalt;
        this.kolor = kolor;
        this.w = w;
        this.h = h;
        this.szer = szer;
        this.liczbaPol = liczbaPol;
        this.len = szer * liczbaPol;
        autos = new int[liczbaPol * 2][liczbaPol * 2];
        autos[0][liczbaPol - 1] = kombStart[0];
        autos[0][liczbaPol] = kombStart[1];
        autos[0][liczbaPol + 1] = kombStart[2];
        for (int m = 1; m < liczbaPol * 2 - 1; m++) {
            for (int n = 1; n < liczbaPol * 2 - 1; n++) {
                int[] arr = new int[3];
                arr[0] = this.autos[m - 1][n - 1];
                arr[1] = this.autos[m - 1][n];
                arr[2] = this.autos[m - 1][n + 1];
                this.autos[m][n] = nextValue(arr, autoNumber);
            }
        }
        this.prefWidth(w);
        this.prefHeight(h);
        this.setWidth(w);
        this.setHeight(h);
        paint();
    }

    private int nextValue(int[] array, int automatNumber) {
        int val = order(array);
        String nbin = Integer.toBinaryString(automatNumber);
        if (nbin.length() < 8) {
            int diff = 8 - nbin.length();
            for (int i = 0; i < diff; i++) {
                nbin = "0".concat(nbin);
            }
        }
        int len = nbin.length();
        String sval = nbin.substring(len - val - 1, len - val);
        return Integer.parseInt(sval);
    }

    private int order(int[] array) {
        int val = 0;
        int len = array.length;
        for (int i = 0; i < len; i++) {
            if (array[len - 1 - i] == 1) {
                val += (int) (Math.pow(2, i));
            }
        }
        return val;
    }

    private void paint() {
        GraphicsContext gc = this.getGraphicsContext2D();
        gc.clearRect(0,0,w,h);
        gc.setStroke(Color.rgb(200, 200, 200));
        int z = 0;// polozenie linii
        for (int i = 0; i <= liczbaPol; i++) {
            gc.strokeLine(0, z, len, z);
            z += szer;
        }
        z = 0;
        for (int k = 0; k <= liczbaPol; k++) {
            gc.strokeLine(z, 0, z, len);
            z += szer;
        }
        gc.setFill(kolor);
        for (int m = 0; m < liczbaPol; m++) {
            for (int j = liczbaPol / 2; j < liczbaPol + liczbaPol / 2; j++) {
                if (autos[m][j] == 1) {
                    int y = m * szer;
                    int x = (j - liczbaPol / 2) * szer;
                    switch (ksztalt) {
                        case KOLO -> gc.fillOval(x, y, szer, szer);
                        case KWADRAT -> gc.fillRect(x, y, szer, szer);
                    }
                }
            }
        }
    }

}
