package autosfx;

import javafx.animation.AnimationTimer;
import javafx.scene.canvas.Canvas;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.paint.Color;

public class LifeCanvas extends Canvas {
    private static final int w = 800;
    private static final int h = 800;
    private static final int liczbaPol = 100;
    private static final int szer = 8;
    private static final int len = szer * liczbaPol;
    private boolean[][] autos1;//komorki aktualne
    private boolean[][] autos2;//komorki nastepnej generacji
    private final Color kolorKomorek;
    private final Ksztalt ksztalt;
    private final GraphicsContext gc;

    public AnimationTimer getTimer() {
        return timer;
    }

    private final AnimationTimer timer;
    private final long millis;
    public LifeCanvas(Ksztalt ksztalt, Color kolorKomorek, double prawdopodobienstwo, long millis) {
        this.kolorKomorek = kolorKomorek;
        //prawdopodobienstwo żywej komórki
        this.ksztalt = ksztalt;
        this.millis = millis;
        this.prefWidth(w);
        this.prefHeight(h);
        this.setWidth(w);
        this.setHeight(h);
        autos1 = new boolean[liczbaPol][liczbaPol];
        autos2 = new boolean[liczbaPol][liczbaPol];
        for (int i = 0; i < liczbaPol; i++) {
            for (int j = 0; j < liczbaPol; j++) {
                autos1[i][j] = Math.random() < prawdopodobienstwo;
                autos2[i][j] = false;
            }
        }
        gc = this.getGraphicsContext2D();
        timer = new LifeTimer();
        timer.start();
    }
    public void paint(GraphicsContext gc) {
        gc.clearRect(0,0,w,h);
        gc.setStroke(Color.rgb(200,200,200));
        int z = 0;//polozenie linii
        for (int i = 0; i <= liczbaPol + 1; i++) {
            gc.strokeLine(0, z, len, z);
            gc.strokeLine(z, 0, z, len);
            z += szer;
        }
        gc.setFill(kolorKomorek);
        for (int i = 0; i < liczbaPol; i++) {
            for (int j = 0; j < liczbaPol; j++) {
                if (autos1[i][j]) {
                    int x = i * szer;
                    int y = j * szer;
                    switch (ksztalt) {
                        case KOLO -> gc.fillOval(x, y, szer, szer);
                        case KWADRAT -> gc.fillRect(x, y, szer-1, szer-1);
                    }
                }
            }
        }
    }

    private int liczbaSasiadow(boolean[][] autos, int p, int q) {
        int x1 = p;
        int x2 = p;
        int y1 = q;
        int y2 = q;
        int licznik = 0;
        if (p > 0) {
            x1--;
        }
        if (p < liczbaPol - 1) {
            x2++;
        }
        if (q > 0) {
            y1--;
        }
        if (q < liczbaPol - 1) {
            y2++;
        }
        for (int i = x1; i <= x2; i++) {
            for (int j = y1; j <= y2; j++) {
                licznik += (autos[i][j]) ? 1 : 0;
            }
        }
        if (autos[p][q]) {
            licznik--;
        }
        return licznik;
    }

    private void clear() {
        for (int j = 0; j < liczbaPol; j++) {
            for (int k = 0; k < liczbaPol; k++) {
                this.autos1[j][k] = false;
                this.autos2[j][k] = false;
            }
        }
    }

    public void putStructure(boolean[][] array) {
        clear();
        for (int m1 = 0; m1 < array.length; m1++) {
            for (int j1 = 0; j1 < array[0].length; j1++) {
                this.autos1[j1 + 25][m1 + 25] = array[m1][j1];
            }
        }
    }

    private void putStructure(boolean[] array, int cols) {
        clear();
        boolean[][] arr = oneToTwo(array, cols);
        for (int m1 = 0; m1 < arr.length; m1++) {
            System.arraycopy(arr[m1], 0, this.autos1[m1 + 25], 25, arr[0].length);
        }
    }

    private boolean[][] oneToTwo(boolean[] tab, int cols) {
        int rows = tab.length / cols;
        boolean[][] temparr = new boolean[rows][cols];
        for (int i = 0; i < tab.length; i++) {
            int x = (i / cols);
            int y = i % cols;
            temparr[x][y] = tab[i];
        }
        return temparr;
    }
    private class LifeTimer extends AnimationTimer {
        @Override
        public void handle(long now) {
            boolean[][] cells = autos1;
            for (int i = 0; i < liczbaPol; i++) {
                for (int j = 0; j < liczbaPol; j++) {
                    autos2[i][j] = cells[i][j];
                    int nb = liczbaSasiadow(cells, i, j);
                    if (nb == 3)
                        autos2[i][j] = true;
                    if (nb < 2 || nb > 3)
                        autos2[i][j] = false;
                }
            }
            autos1 = autos2;
            autos2 = cells;
            paint(gc);
            try {
                Thread.sleep(millis);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }
}
